// SPDX-License-Identifier: CC0-1.0
//
// SPDX-FileContributor: Antonio Niño Díaz, 2024

#include <nds.h>

// This is autogenerated from ball.png and ball.grit
#include "ball.h"

int main(int argc, char *argv[])
{
    videoSetMode(MODE_0_2D);
    videoSetModeSub(MODE_0_2D);

    // VRAM_C is used by consoleDemoInit()
    vramSetPrimaryBanks(VRAM_A_MAIN_SPRITE, VRAM_B_LCD, VRAM_C_LCD, VRAM_D_SUB_SPRITE);

    consoleDemoInit();

    // Initialize sprites with extended palettes active
    oamInit(&oamMain, SpriteMapping_Bmp_1D_128, true);
    oamInit(&oamSub, SpriteMapping_Bmp_1D_128, true);

    // Allocate space for the tiles and copy them there
    u16 *gfx256ExtMain = oamAllocateGfx(&oamMain, SpriteSize_32x32, SpriteColorFormat_256Color);
    dmaCopy(ballTiles, gfx256ExtMain, ballTilesLen);

    u16 *gfx256ExtSub = oamAllocateGfx(&oamSub, SpriteSize_32x32, SpriteColorFormat_256Color);
    dmaCopy(ballTiles, gfx256ExtSub, ballTilesLen);

    const int spr_pal_main = 3;
    const int spr_pal_sub = 4;

    // Map VRAM as LCD because it can't be accessed by the CPU while it is
    // mapped as extended palette VRAM.
    vramSetBankF(VRAM_F_LCD);
    vramSetBankI(VRAM_I_LCD);

    // Copy palettes
    dmaCopy(ballPal, VRAM_F_EXT_SPR_PALETTE[spr_pal_main], ballPalLen);
    dmaCopy(ballPal, VRAM_I_EXT_SPR_PALETTE[spr_pal_sub], ballPalLen);

    // Map some VRAM to be used as extended palettes
    vramSetBankF(VRAM_F_SPRITE_EXT_PALETTE);
    vramSetBankI(VRAM_I_SUB_SPRITE_EXT_PALETTE);

    oamSet(&oamMain, 0,
           150, 70, // X, Y
           0, // Priority
           spr_pal_main, // Palette index
           SpriteSize_32x32, SpriteColorFormat_256Color, // Size, format
           gfx256ExtMain, // Graphics offset
           -1, // Affine index
           false, // Double size
           false, // Hide
           false, false, // H flip, V flip
           false); // Mosaic

    oamSet(&oamSub, 0,
           10, 70, // X, Y
           0, // Priority
           spr_pal_sub, // Palette index
           SpriteSize_32x32, SpriteColorFormat_256Color, // Size, format
           gfx256ExtSub, // Graphics offset
           -1, // Affine index
           false, // Double size
           false, // Hide
           false, false, // H flip, V flip
           false); // Mosaic

    printf("PAD:   Move sprite\n");
    printf("START: Exit to loader\n");

    int x = 150, y = 70;

    while (1)
    {
        swiWaitForVBlank();

        oamSetXY(&oamMain, 0, x, y);

        oamUpdate(&oamMain);
        oamUpdate(&oamSub);

        scanKeys();

        u16 keys_held = keysHeld();

        if (keys_held & KEY_UP)
            y--;
        else if (keys_held & KEY_DOWN)
            y++;

        if (keys_held & KEY_LEFT)
            x--;
        else if (keys_held & KEY_RIGHT)
            x++;

        if (keys_held & KEY_START)
            break;
    }

    oamFreeGfx(&oamMain, gfx256ExtMain);
    oamFreeGfx(&oamSub, gfx256ExtSub);

    return 0;
}
